pub mod app;
pub mod formats;
pub mod utils;

use std::sync::Arc;

use crate::{
    app::{App, Config},
    utils::error::Result,
};

#[tokio::main]
async fn main() -> Result<()> {
    let config = Config::from_env()?;

    println!("== config");
    println!("{config:?}");
    println!("==");

    let app = Arc::new(App::new(config)?);
    let addr = app.addr.clone();

    let router = app.into_router();

    println!("Run on {}", addr);
    let listener = tokio::net::TcpListener::bind(addr).await?;
    axum::serve(listener, router).await?;

    Ok(())
}
