use std::backtrace::{Backtrace, BacktraceStatus};

pub type Result<T, E = Error> = std::result::Result<T, E>;

pub struct Error(pub String, pub Backtrace);

trait Convertible: std::fmt::Display {}

impl<E: std::error::Error> From<E> for Error {
    fn from(value: E) -> Self {
        Error(value.to_string(), Backtrace::capture())
    }
}

impl std::fmt::Display for Error {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self.1.status() {
            BacktraceStatus::Captured => write!(f, "{}\nBacktrace:\n{}", self.0, self.1),
            _ => write!(f, "{}", self.0),
        }
    }
}

impl std::fmt::Debug for Error {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "{self}")
    }
}

macro_rules! error {
    ($($tt:tt)*) => {
        $crate::utils::error::Error(format!($($tt)*), std::backtrace::Backtrace::capture())
    };
}

pub(crate) use error;

macro_rules! fail {
    ($($tt:tt)*) => {
        return Err($crate::utils::error::error!($($tt)*))
    };
}

pub(crate) use fail;
